@@ -3,8 +3,9 @@ import sys
 
 sys.path.append(str(pathlib.Path(__file__).parent.parent))
 
-from multi_file.app import MyApp  # noqa: E402
-from qtpy.QtWidgets import QApplication  # noqa: E402
+from qtpy.QtWidgets import QApplication
+
+from multi_file.app import MyApp
 
 qapp = QApplication.instance() or QApplication([])
 app = MyApp()
