@@ -1,44 +1,57 @@
+# mypy: disable-error-code="operator"
+from __future__ import annotations
+
 import operator
 from functools import reduce
-from typing import Dict, Optional, Union, cast
+from typing import TYPE_CHECKING, Mapping, MutableMapping
 
+from qtpy import API, QT_VERSION
 from qtpy.QtCore import QCoreApplication, Qt
 from qtpy.QtGui import QKeySequence
 
-from ...types._constants import OperatingSystem
-from ...types._keys import KeyBinding, KeyCode, KeyCombo, KeyMod, SimpleKeyBinding
+from app_model.types import (
+    KeyBinding,
+    KeyCode,
+    KeyCombo,
+    KeyMod,
+    SimpleKeyBinding,
+)
+from app_model.types._constants import OperatingSystem
+
+if TYPE_CHECKING:
+    from qtpy.QtCore import QKeyCombination
 
 try:
     from qtpy import QT6
 except ImportError:
     QT6 = False
 
-QMETA = Qt.KeyboardModifier.MetaModifier
 QCTRL = Qt.KeyboardModifier.ControlModifier
+QSHIFT = Qt.KeyboardModifier.ShiftModifier
+QALT = Qt.KeyboardModifier.AltModifier
+QMETA = Qt.KeyboardModifier.MetaModifier
 
 MAC = OperatingSystem.current().is_mac
 
 _QMOD_LOOKUP = {
     "ctrl": QCTRL,
-    "shift": Qt.KeyboardModifier.ShiftModifier,
-    "alt": Qt.KeyboardModifier.AltModifier,
+    "shift": QSHIFT,
+    "alt": QALT,
     "meta": QMETA,
 }
 
-
-def mac_ctrl_meta_swapped() -> bool:
-    """Whether or not Qt has swapped ctrl and meta for Macs."""
-    return not QCoreApplication.testAttribute(Qt.AA_MacDontSwapCtrlAndMeta)
+_SWAPPED_QMOD_LOOKUP = {
+    **_QMOD_LOOKUP,
+    "ctrl": QMETA,
+    "meta": QCTRL,
+}
 
 
-def _mac_qmod_lookup() -> Dict[str, Qt.KeyboardModifier]:
-    swapped = mac_ctrl_meta_swapped()
-    return {
-        "ctrl": QMETA if swapped else QCTRL,
-        "shift": Qt.KeyboardModifier.ShiftModifier,
-        "alt": Qt.KeyboardModifier.AltModifier,
-        "meta": QCTRL if swapped else QMETA,
-    }
+def _mac_ctrl_meta_swapped() -> bool:
+    """Return True if Qt is swapping Ctrl and Meta for keyboard interactions."""
+    return not QCoreApplication.testAttribute(
+        Qt.ApplicationAttribute.AA_MacDontSwapCtrlAndMeta
+    )
 
 
 if QT6:
@@ -46,25 +59,30 @@ if QT6:
 
     def simple_keybinding_to_qint(skb: SimpleKeyBinding) -> int:
         """Create Qt Key integer from a SimpleKeyBinding."""
-        lookup = _mac_qmod_lookup() if MAC else _QMOD_LOOKUP
-        key = KEY_TO_QT.get(skb.key, 0)
+        lookup = (
+            _SWAPPED_QMOD_LOOKUP if MAC and _mac_ctrl_meta_swapped() else _QMOD_LOOKUP
+        )
+        key = modelkey2qkey(skb.key) if skb.key else Qt.Key.Key_unknown
         mods = (v for k, v in lookup.items() if getattr(skb, k))
-        combo = QKeyCombination(reduce(operator.or_, mods), key)
-        return cast(int, combo.toCombined())
+        combo = QKeyCombination(
+            reduce(operator.or_, mods, Qt.KeyboardModifier.NoModifier), key
+        )
+        return int(combo.toCombined())
 
 else:
-    QKeyCombination = int
 
     def simple_keybinding_to_qint(skb: SimpleKeyBinding) -> int:
         """Create Qt Key integer from a SimpleKeyBinding."""
-        lookup = _mac_qmod_lookup() if MAC else _QMOD_LOOKUP
-        out = KEY_TO_QT.get(skb.key, 0)
+        lookup = (
+            _SWAPPED_QMOD_LOOKUP if MAC and _mac_ctrl_meta_swapped() else _QMOD_LOOKUP
+        )
+        out = modelkey2qkey(skb.key) if skb.key else 0
         mods = (v for k, v in lookup.items() if getattr(skb, k))
         out = reduce(operator.or_, mods, out)
         return int(out)
 
 
-if QT6:
+if QT6 and not (API == "pyside6" and int(QT_VERSION[2]) < 4):
 
     def _get_qmods(key: QKeyCombination) -> Qt.KeyboardModifier:
         return key.keyboardModifiers()
@@ -92,7 +110,7 @@ class QKeyBindingSequence(QKeySequence):
         super().__init__(*ints)
 
 
-KEY_TO_QT: Dict[Optional[KeyCode], Qt.Key] = {
+KEY_TO_QT: dict[KeyCode | None, Qt.Key] = {
     None: Qt.Key.Key_unknown,
     KeyCode.UNKNOWN: Qt.Key.Key_unknown,
     KeyCode.Backquote: Qt.Key.Key_QuoteLeft,
@@ -197,24 +215,35 @@ KEY_TO_QT: Dict[Optional[KeyCode], Qt.Key] = {
     KeyCode.PauseBreak: Qt.Key.Key_Pause,
 }
 
-KEYMOD_TO_QT = {
-    KeyMod.NONE: Qt.KeyboardModifier.NoModifier,
-    KeyMod.Alt: Qt.KeyboardModifier.AltModifier,
-    KeyMod.CtrlCmd: Qt.KeyboardModifier.ControlModifier,
-    KeyMod.Shift: Qt.KeyboardModifier.ShiftModifier,
-    KeyMod.WinCtrl: Qt.KeyboardModifier.MetaModifier,
+KEYMOD_FROM_QT = {
+    Qt.KeyboardModifier.NoModifier: KeyMod.NONE,
+    QALT: KeyMod.Alt,
+    QCTRL: KeyMod.CtrlCmd,
+    QSHIFT: KeyMod.Shift,
+    QMETA: KeyMod.WinCtrl,
 }
 
+MAC_KEYMOD_FROM_QT = {**KEYMOD_FROM_QT, QCTRL: KeyMod.WinCtrl, QMETA: KeyMod.CtrlCmd}
 
-# unused/untested
-KEY_FROM_QT: Dict[Qt.Key, KeyCode] = {
+KEYMOD_TO_QT = {
+    KeyMod.NONE: Qt.KeyboardModifier.NoModifier,
+    KeyMod.CtrlCmd: QCTRL,
+    KeyMod.Alt: QALT,
+    KeyMod.Shift: QSHIFT,
+    KeyMod.WinCtrl: QMETA,
+}
+
+MAC_KEYMOD_TO_QT = {**KEYMOD_TO_QT, KeyMod.WinCtrl: QCTRL, KeyMod.CtrlCmd: QMETA}
+
+
+KEY_FROM_QT: MutableMapping[Qt.Key, KeyCode | KeyCombo] = {
     v.toCombined() if hasattr(v, "toCombined") else int(v): k
     for k, v in KEY_TO_QT.items()
     if k
 }
 
 # Qt Keys which have no representation in the W3C spec
-_QTONLY_KEYS = {
+_QTONLY_KEYS: Mapping[Qt.Key, KeyCode | KeyCombo] = {
     Qt.Key.Key_Exclam: KeyMod.Shift | KeyCode.Digit1,
     Qt.Key.Key_At: KeyMod.Shift | KeyCode.Digit2,
     Qt.Key.Key_NumberSign: KeyMod.Shift | KeyCode.Digit3,
@@ -244,48 +273,51 @@ KEY_FROM_QT.update(_QTONLY_KEYS)
 
 def qmods2modelmods(modifiers: Qt.KeyboardModifier) -> KeyMod:
     """Return KeyMod from Qt.KeyboardModifier."""
-    swapped = None
     mod = KeyMod.NONE
-    if modifiers & Qt.KeyboardModifier.ShiftModifier:
-        mod |= KeyMod.Shift
-    if modifiers & Qt.KeyboardModifier.ControlModifier:
-        swapped = MAC and mac_ctrl_meta_swapped()
-        if swapped:
-            mod |= KeyMod.WinCtrl
-        else:
-            mod |= KeyMod.CtrlCmd
-    if modifiers & Qt.KeyboardModifier.AltModifier:
-        mod |= KeyMod.Alt
-    if modifiers & Qt.KeyboardModifier.MetaModifier:
-        if swapped is None:
-            swapped = MAC and mac_ctrl_meta_swapped()
-        if swapped:
-            mod |= KeyMod.CtrlCmd
-        else:
-            mod |= KeyMod.WinCtrl
+    lookup = (
+        MAC_KEYMOD_FROM_QT if MAC and not _mac_ctrl_meta_swapped() else KEYMOD_FROM_QT
+    )
+    for modifier in lookup:
+        if modifiers & modifier:
+            mod |= lookup[modifier]
     return mod
 
 
-def qkey2modelkey(key: Qt.Key) -> KeyCode:
+def modelkey2qkey(key: KeyCode) -> Qt.Key:
+    """Return Qt.Key from KeyCode."""
+    if MAC and _mac_ctrl_meta_swapped():
+        if key == KeyCode.Meta:
+            return Qt.Key.Key_Control
+        if key == KeyCode.Ctrl:
+            return Qt.Key.Key_Meta
+    return KEY_TO_QT.get(key, Qt.Key.Key_unknown)
+
+
+def qkey2modelkey(key: Qt.Key) -> KeyCode | KeyCombo:
     """Return KeyCode from Qt.Key."""
+    if MAC and _mac_ctrl_meta_swapped():
+        if key == Qt.Key.Key_Control:
+            return KeyCode.Meta
+        if key == Qt.Key.Key_Meta:
+            return KeyCode.Ctrl
     return KEY_FROM_QT.get(key, KeyCode.UNKNOWN)
 
 
-def qkeycombo2modelkey(key: QKeyCombination) -> Union[KeyCode, KeyCombo]:
+def qkeycombo2modelkey(key: QKeyCombination) -> KeyCode | KeyCombo:
     """Return KeyCode or KeyCombo from QKeyCombination."""
     if key in KEY_FROM_QT:
+        # type ignore because in qt5, key may actually just be int ... but it's fine.
         return KEY_FROM_QT[key]
     qmods = _get_qmods(key)
     qkey = _get_qkey(key)
-    return qmods2modelmods(qmods) | qkey2modelkey(qkey)
+    return qmods2modelmods(qmods) | qkey2modelkey(qkey)  # type: ignore [return-value]
 
 
 def qkeysequence2modelkeybinding(key: QKeySequence) -> KeyBinding:
     """Return KeyBinding from QKeySequence."""
     # FIXME: this should return KeyChord instead of KeyBinding... but that only takes 2
-    return KeyBinding(
-        parts=[SimpleKeyBinding.from_int(qkeycombo2modelkey(x)) for x in key]
-    )
+    parts = [SimpleKeyBinding.from_int(qkeycombo2modelkey(x)) for x in iter(key)]
+    return KeyBinding(parts=parts)
 
 
 # ################# These are the Qkeys we currently aren't mapping ################ #
