@@ -1,28 +1,5 @@
-"""Provides the  :class:`Expr` and its subclasses.
+"""Provides `Expr` and its subclasses."""
 
-`Expr` is a subclass of `ast.AST` that provides rich dunder methods that
-facilitate joining and comparing typed expressions.  It only implements a
-subset of ast Expressions (for safety of evaluation), but provides more
-than ast.literal_eval.
-
-Expressions that are supported:
-- Names: 'myvar' (these must be evaluated along with some context)
-- Constants: '1'
-- Comparisons: 'myvar > 1'
-- Boolean Operators: 'myvar and yourvar'
-- Binary Operators: 'myvar + 42'
-- Unary Operators: 'not myvar'
-
-Things that are *NOT* supported:
-- attribute access: 'my.attr'
-- calls: 'f(x)'
-- containers (lists, tuples, sets, dicts)
-- indexing or slicing
-- joined strings (f-strings)
-- named expressions (walrus operator)
-- comprehensions (list, set, dict, generator)
-- statements & assignments (e.g. 'a = b')
-"""
 from __future__ import annotations
 
 import ast
@@ -30,15 +7,11 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
     Generic,
     Iterator,
-    List,
     Mapping,
-    Optional,
     Sequence,
     SupportsIndex,
-    Tuple,
     Type,
     TypeVar,
     Union,
@@ -56,16 +29,21 @@ T2 = TypeVar("T2", bound=Union[ConstType, "Expr"])
 V = TypeVar("V", bound=ConstType)
 
 if TYPE_CHECKING:
+    from types import CodeType
+
+    from pydantic.annotated_handlers import GetCoreSchemaHandler
+    from pydantic_core import core_schema
+
     from ._context_keys import ContextKey
 
 
-def parse_expression(expr: Union[str, Expr]) -> Expr:
-    """Parse string expression into an :class:`Expr` instance.
+def parse_expression(expr: Expr | str) -> Expr:
+    """Parse string expression into an [`Expr`][app_model.expressions.Expr] instance.
 
     Parameters
     ----------
-    expr : Union[str, Expr]
-        Expression to parse.  (If already an :class:`Expr`, it is returned)
+    expr : Expr | str
+        Expression to parse.  (If already an `Expr`, it is returned)
 
     Returns
     -------
@@ -87,12 +65,12 @@ def parse_expression(expr: Union[str, Expr]) -> Expr:
         tree = ast.parse(str(expr), mode="eval")
         if not isinstance(tree, ast.Expression):
             raise SyntaxError  # pragma: no cover
-        return ExprTranformer().visit(tree.body)
+        return ExprTransformer().visit(tree.body)
     except SyntaxError as e:
         raise SyntaxError(f"{expr!r} is not a valid expression: ({e}).") from None
 
 
-def safe_eval(expr: Union[str, bool, Expr], context: Optional[Mapping] = None) -> Any:
+def safe_eval(expr: str | bool | Expr, context: Mapping | None = None) -> Any:
     """Safely evaluate `expr` string given `context` dict.
 
     This lets you evaluate a string expression with broader expression
@@ -102,20 +80,46 @@ def safe_eval(expr: Union[str, bool, Expr], context: Optional[Mapping] = None) -
 
     Parameters
     ----------
-    expr : Union[str, bool, Expr]
+    expr : str | bool | Expr
         Expression to evaluate. If `expr` is a string, it is parsed into an
-        :class:`Expr` instance. If a `bool`, it is returned directly.
-    context : Optional[Mapping]
+        `Expr` instance. If a `bool`, it is returned directly.
+    context : Mapping | None
         Context (mapping of names to objects) to evaluate the expression in.
     """
     if isinstance(expr, bool):
         return expr
-    return parse_expression(expr).eval(context or {})
+    return parse_expression(expr).eval(context)
 
 
 class Expr(ast.AST, Generic[T]):
     """Base Expression class providing dunder and convenience methods.
 
+    This is a subclass of `ast.AST` that provides rich dunder methods that
+    facilitate joining and comparing typed expressions.  It only implements a
+    subset of ast Expressions (for safety of evaluation), but provides more
+    than `ast.literal_eval`.
+
+    Expressions that are supported:
+
+    - Names: 'myvar' (these must be evaluated along with some context)
+    - Constants: '1'
+    - Comparisons: 'myvar > 1'
+    - Boolean Operators: 'myvar & yourvar' (bitwise `&` and `|` are overloaded
+      here to mean boolean `and` and `or`)
+    - Binary Operators: 'myvar + 42' (includes `//`, `@`, `^`)
+    - Unary Operators: 'not myvar'
+
+    Things that are *NOT* supported:
+
+    - attribute access: 'my.attr'
+    - calls: 'f(x)'
+    - containers (lists, tuples, sets, dicts)
+    - indexing or slicing
+    - joined strings (f-strings)
+    - named expressions (walrus operator)
+    - comprehensions (list, set, dict, generator)
+    - statements & assignments (e.g. 'a = b')
+
     This class is not meant to be instantiated directly. Instead, use
     [`parse_expression`][app_model.expressions._expressions.parse_expression], or the
     [`Expr.parse`][app_model.expressions.Expr.parse] classmethod to create an expression
@@ -127,10 +131,10 @@ class Expr(ast.AST, Generic[T]):
 
     Examples
     --------
-    >>> expr = parse_expression('myvar > 5')
+    >>> expr = parse_expression("myvar > 5")
 
     combine expressions with operators
-    >>> new_expr = expr & parse_expression('v2')
+    >>> new_expr = expr & parse_expression("v2")
 
     nice `repr`
     >>> new_expr
@@ -146,9 +150,12 @@ class Expr(ast.AST, Generic[T]):
             Name(id='v2', ctx=Load())])
 
     evaluate in some context
-    >>> new_expr.eval(dict(v2='hello!', myvar=8))
+    >>> new_expr.eval(dict(v2="hello!", myvar=8))
     'hello!'
 
+    you can also use keyword arguments.  This is *slightly* slower
+    >>> new_expr.eval(v2="hello!", myvar=4)
+
     serialize
     >>> str(new_expr)
     'myvar > 5 and v2'
@@ -157,34 +164,45 @@ class Expr(ast.AST, Generic[T]):
     that can be evaluated repeatedly as some underlying context changes.
 
     ```python
-    light_is_green = Name[bool]('light_is_green')
-    count = Name[int]('count')
+    light_is_green = Name[bool]("light_is_green")
+    count = Name[int]("count")
     is_ready = light_is_green & count > 5
 
-    assert is_ready.eval({'count': 4, 'light_is_green': True}) == False
-    assert is_ready.eval({'count': 7, 'light_is_green': False}) == False
-    assert is_ready.eval({'count': 7, 'light_is_green': True}) == True
+    assert is_ready.eval({"count": 4, "light_is_green": True}) == False
+    assert is_ready.eval({"count": 7, "light_is_green": False}) == False
+    assert is_ready.eval({"count": 7, "light_is_green": True}) == True
     ```
 
     this will also preserve type information:
     >>> reveal_type(is_ready())  # revealed type is `bool`
     """
 
+    _names: set[str]
+    _code: CodeType
+
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         if type(self).__name__ == "Expr":
             raise RuntimeError("Don't instantiate Expr. Use `Expr.parse`")
         super().__init__(*args, **kwargs)
-        ast.fix_missing_locations(self)
+        self._recompile()
 
-    def eval(self, context: Optional[Mapping[str, object]] = None) -> T:
+    def _recompile(self) -> None:
+        ast.fix_missing_locations(self)
+        self._code = compile(ast.Expression(body=self), "<Expr>", "eval")  # type: ignore [arg-type]
+        self._names = set(self._iter_names())
+
+    def eval(
+        self, context: Mapping[str, object] | None = None, **ctx_kwargs: object
+    ) -> T:
         """Evaluate this expression with names in `context`."""
         if context is None:
-            context = {}
-        code = compile(ast.Expression(body=self), "<Expr>", "eval")
+            context = ctx_kwargs
+        elif ctx_kwargs:
+            context = {**context, **ctx_kwargs}
         try:
-            return cast(T, eval(code, {}, context))
+            return eval(self._code, {}, context)  # type: ignore
         except NameError as e:
-            miss = {k for k in _iter_names(self) if k not in context}
+            miss = {k for k in self._names if k not in context}
             raise NameError(
                 f"Names required to eval this expression are missing: {miss}"
             ) from e
@@ -215,18 +233,16 @@ class Expr(ast.AST, Generic[T]):
         return obj if isinstance(obj, Expr) else Constant(obj)
 
     # boolean operators
-    # '&' and '|' are normaly binary operators... but we use them here to
+    # '&' and '|' are normally binary operators... but we use them here to
     # combine expression objects meaning "and" and "or".
     # if you want the binary operators, use Expr.bitand, and Expr.bitor
 
     def __and__(
-        self, other: Union[Expr[T2], Expr[T], ConstType, Compare]
-    ) -> BoolOp[Union[T, T2]]:
+        self, other: Expr[T2] | Expr[T] | ConstType | Compare
+    ) -> BoolOp[T | T2]:
         return BoolOp(ast.And(), [self, other])
 
-    def __or__(
-        self, other: Union[Expr[T2], Expr[T], ConstType, Compare]
-    ) -> BoolOp[Union[T, T2]]:
+    def __or__(self, other: Expr[T2] | Expr[T] | ConstType | Compare) -> BoolOp[T | T2]:
         return BoolOp(ast.Or(), [self, other])
 
     # comparisons
@@ -264,38 +280,38 @@ class Expr(ast.AST, Generic[T]):
     # binary operators
     # (note that __and__ and __or__ are reserved for boolean operators.)
 
-    def __add__(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def __add__(self, other: T | Expr[T]) -> BinOp[T]:
         return BinOp(self, ast.Add(), other)
 
-    def __sub__(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def __sub__(self, other: T | Expr[T]) -> BinOp[T]:
         return BinOp(self, ast.Sub(), other)
 
-    def __mul__(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def __mul__(self, other: T | Expr[T]) -> BinOp[T]:
         return BinOp(self, ast.Mult(), other)
 
-    def __truediv__(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def __truediv__(self, other: T | Expr[T]) -> BinOp[T]:
         return BinOp(self, ast.Div(), other)
 
-    def __floordiv__(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def __floordiv__(self, other: T | Expr[T]) -> BinOp[T]:
         return BinOp(self, ast.FloorDiv(), other)
 
-    def __mod__(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def __mod__(self, other: T | Expr[T]) -> BinOp[T]:
         return BinOp(self, ast.Mod(), other)
 
-    def __matmul__(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def __matmul__(self, other: T | Expr[T]) -> BinOp[T]:
         return BinOp(self, ast.MatMult(), other)  # pragma: no cover
 
-    def __pow__(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def __pow__(self, other: T | Expr[T]) -> BinOp[T]:
         return BinOp(self, ast.Pow(), other)
 
-    def __xor__(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def __xor__(self, other: T | Expr[T]) -> BinOp[T]:
         return BinOp(self, ast.BitXor(), other)
 
-    def bitand(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def bitand(self, other: T | Expr[T]) -> BinOp[T]:
         """Return bitwise self & other."""
         return BinOp(self, ast.BitAnd(), other)
 
-    def bitor(self, other: Union[T, Expr[T]]) -> BinOp[T]:
+    def bitor(self, other: T | Expr[T]) -> BinOp[T]:
         """Return bitwise self | other."""
         return BinOp(self, ast.BitOr(), other)
 
@@ -312,7 +328,7 @@ class Expr(ast.AST, Generic[T]):
         # note: we're using the invert operator `~` to mean "not ___"
         return UnaryOp(ast.Not(), self)
 
-    def __reduce_ex__(self, protocol: SupportsIndex) -> Tuple[Any, ...]:
+    def __reduce_ex__(self, protocol: SupportsIndex) -> tuple[Any, ...]:
         rv = list(super().__reduce_ex__(protocol))
         rv[1] = tuple(getattr(self, f) for f in self._fields)
         return tuple(rv)
@@ -320,11 +336,20 @@ class Expr(ast.AST, Generic[T]):
     @classmethod
     def __get_validators__(cls) -> Iterator[Callable[[Any], Expr]]:
         """Pydantic validators for this class."""
-        yield cls.validate
+        yield cls._validate
 
     @classmethod
-    def validate(cls, v: Any) -> Expr:
-        """Validator for Expr. For use with Pydantic."""
+    def __get_pydantic_core_schema__(
+        cls, source: type, handler: GetCoreSchemaHandler
+    ) -> core_schema.CoreSchema:
+        # this will only be called by pydantic v2
+        from pydantic_core import core_schema
+
+        return core_schema.no_info_plain_validator_function(cls._validate)
+
+    @classmethod
+    def _validate(cls, v: Any) -> Expr:
+        """Validate v as an `Expr`. For use with Pydantic."""
         return v if isinstance(v, Expr) else parse_expression(v)
 
     def __hash__(self) -> int:
@@ -336,6 +361,9 @@ class Expr(ast.AST, Generic[T]):
             _hash += hash(field)
         return _hash
 
+    def _iter_names(self) -> Iterator[str]:
+        yield from _iter_names(self)
+
 
 LOAD = ast.Load()
 
@@ -350,12 +378,6 @@ class Name(Expr[T], ast.Name):
         kwargs["ctx"] = LOAD
         super().__init__(id, **kwargs)
 
-    def eval(self, context: Optional[Mapping] = None) -> T:
-        """Evaluate this expression with names in `context`."""
-        if context is None:
-            context = {}
-        return super().eval(context=context)
-
 
 class Constant(Expr[V], ast.Constant):
     """A constant value.
@@ -366,7 +388,7 @@ class Constant(Expr[V], ast.Constant):
 
     value: V
 
-    def __init__(self, value: V, kind: Optional[str] = None, **kwargs: Any) -> None:
+    def __init__(self, value: V, kind: str | None = None, **kwargs: Any) -> None:
         _valid_type = (type(None), str, bytes, bool, int, float)
         if not isinstance(value, _valid_type):
             raise TypeError(f"Constants must be type: {_valid_type!r}")
@@ -404,9 +426,9 @@ class BinOp(Expr[T], ast.BinOp):
 
     def __init__(
         self,
-        left: Union[T, Expr[T]],
+        left: T | Expr[T],
         op: ast.operator,
-        right: Union[T, Expr[T]],
+        right: T | Expr[T],
         **k: Any,
     ) -> None:
         super().__init__(Expr._cast(left), op, Expr._cast(right), **k)
@@ -419,13 +441,13 @@ class BoolOp(Expr[T], ast.BoolOp):
     with the same operator, such as a or b or c, are collapsed into one node
     with several values.
 
-    This doesn't include `not`, which is a :class:`UnaryOp`.
+    This doesn't include `not`, which is a `UnaryOp`.
     """
 
     def __init__(
         self,
         op: ast.boolop,
-        values: Sequence[Union[ConstType, Expr]],
+        values: Sequence[ConstType | Expr],
         **kwargs: Any,
     ):
         super().__init__(op, [Expr._cast(v) for v in values], **kwargs)
@@ -453,13 +475,52 @@ class IfExp(Expr, ast.IfExp):
         )
 
 
-class ExprTranformer(ast.NodeTransformer):
-    """Transformer that converts an ast.expr into an :class:`Expr`.
+class Tuple(Expr, ast.Tuple):
+    """A tuple expression.
+
+    `elts` is a list of expressions.
+    """
+
+    def __init__(
+        self, elts: Sequence[Expr], ctx: ast.expr_context = LOAD, **kwargs: Any
+    ) -> None:
+        kwargs["ctx"] = ctx
+        super().__init__(elts=[Expr._cast(e) for e in elts], **kwargs)
+
+
+class List(Expr, ast.List):
+    """A tuple expression.
+
+    `elts` is a list of expressions.
+    """
+
+    def __init__(
+        self, elts: Sequence[Expr], ctx: ast.expr_context = LOAD, **kwargs: Any
+    ) -> None:
+        kwargs["ctx"] = ctx
+        super().__init__(elts=[Expr._cast(e) for e in elts], **kwargs)
+
+
+class Set(Expr, ast.Set):
+    """A tuple expression.
+
+    `elts` is a list of expressions.
+    """
+
+    def __init__(
+        self, elts: Sequence[Expr], ctx: ast.expr_context = LOAD, **kwargs: Any
+    ) -> None:
+        kwargs["ctx"] = ctx
+        super().__init__(elts=[Expr._cast(e) for e in elts], **kwargs)
+
+
+class ExprTransformer(ast.NodeTransformer):
+    """Transformer that converts an ast.expr into an `Expr`.
 
     Examples
     --------
-    >>> tree = ast.parse('my_var > 11', mode='eval')
-    >>> tree = ExprTranformer().visit(tree)  # transformed
+    >>> tree = ast.parse("my_var > 11", mode="eval")
+    >>> tree = ExprTransformer().visit(tree)  # transformed
     """
 
     _SUPPORTED_NODES = frozenset(
@@ -468,12 +529,11 @@ class ExprTranformer(ast.NodeTransformer):
 
     # fmt: off
     @overload
-    def visit(self, node: ast.expr) -> Expr: ...  # noqa
+    def visit(self, node: ast.expr) -> Expr: ...
     @overload
-    def visit(self, node: PassedType) -> PassedType: ...  # noqa
+    def visit(self, node: PassedType) -> PassedType: ...
     # fmt: on
-
-    def visit(self, node: ast.AST) -> Optional[ast.AST]:
+    def visit(self, node: ast.AST) -> ast.AST | None:
         """Visit a node in the tree, transforming into Expr."""
         if isinstance(
             node,
@@ -491,12 +551,12 @@ class ExprTranformer(ast.NodeTransformer):
         # filter here for supported expression node types
         type_ = type(node).__name__
 
-        if type_ not in ExprTranformer._SUPPORTED_NODES:
+        if type_ not in ExprTransformer._SUPPORTED_NODES:
             raise SyntaxError(f"Type {type_!r} not supported")
 
         # providing fake lineno and col_offset here rather than using
         # ast.fill_missing_locations for typing purposes
-        kwargs: Dict[str, Any] = dict(lineno=1, col_offset=0)
+        kwargs: dict[str, Any] = {"lineno": 1, "col_offset": 0}
 
         for name, field in ast.iter_fields(node):
             if isinstance(field, ast.expr):
@@ -511,25 +571,25 @@ class ExprTranformer(ast.NodeTransformer):
 
 
 class _ExprSerializer(ast.NodeVisitor):
-    """Serializes an :class:`Expr` into a string.
+    """Serializes an `Expr` into a string.
 
     Examples
     --------
-    >>> expr = Expr.parse('a + b == c')
+    >>> expr = Expr.parse("a + b == c")
     >>> print(expr)
     'a + b == c'
 
     or ... using this visitor directly:
 
-    >>> ser = ExprSerializer()
-    >>> ser.visit(expr)
-    >>> out = "".join(ser.result)
+    >>> serializer = ExprSerializer()
+    >>> serializer.visit(expr)
+    >>> out = "".join(serializer.result)
     """
 
-    def __init__(self, node: Optional[Expr] = None) -> None:
-        self._result: List[str] = []
+    def __init__(self, node: Expr | None = None) -> None:
+        self._result: list[str] = []
 
-        def write(*params: Union[ast.AST, str]) -> None:
+        def write(*params: ast.AST | str) -> None:
             for item in params:
                 if isinstance(item, ast.AST):
                     self.visit(item)
@@ -547,6 +607,15 @@ class _ExprSerializer(ast.NodeVisitor):
     def visit_Name(self, node: ast.Name) -> None:
         self.write(node.id)
 
+    def visit_Tuple(self, node: ast.Tuple) -> None:
+        self.write(f'({", ".join(map(str, node.elts))})')
+
+    def visit_Set(self, node: ast.Set) -> None:
+        self.write("{" + ", ".join(map(str, node.elts)) + "}")
+
+    def visit_List(self, node: ast.List) -> None:
+        self.write(f'[{", ".join(map(str, node.elts))}]')
+
     def visit_ContextKey(self, node: ContextKey) -> None:
         return self.visit_Name(node)
 
@@ -575,7 +644,7 @@ class _ExprSerializer(ast.NodeVisitor):
 
 
 OpType = Union[Type[ast.operator], Type[ast.cmpop], Type[ast.boolop], Type[ast.unaryop]]
-_OPS: Dict[OpType, str] = {
+_OPS: dict[OpType, str] = {
     # ast.boolop
     ast.Or: "or",
     ast.And: "and",
