@@ -1,23 +1,13 @@
 from __future__ import annotations
 
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Final,
-    Iterable,
-    Iterator,
-    List,
-    Optional,
-    Sequence,
-    Set,
-    Tuple,
-)
+from typing import TYPE_CHECKING, Any, Callable, Final, Iterable, Iterator
 
 from psygnal import Signal
 
-from ..types import MenuItem, MenuOrSubmenu
-from ..types._constants import DisposeCallable
+from app_model.types import MenuItem
+
+if TYPE_CHECKING:
+    from app_model.types import Action, DisposeCallable, MenuOrSubmenu
 
 MenuId = str
 
@@ -29,16 +19,56 @@ class MenusRegistry:
     menus_changed = Signal(set)
 
     def __init__(self) -> None:
-        self._menu_items: Dict[MenuId, Dict[MenuOrSubmenu, None]] = {}
+        self._menu_items: dict[MenuId, dict[MenuOrSubmenu, None]] = {}
+
+    def append_action_menus(self, action: Action) -> DisposeCallable | None:
+        """Append all MenuRule items declared in `action.menus`.
+
+        Parameters
+        ----------
+        action : Action
+            The action containing menus to append.
+
+        Returns
+        -------
+        DisposeCallable | None
+            A function that can be called to unregister the menu items. If no
+            menu items were registered, returns `None`.
+        """
+        disposers: list[Callable[[], None]] = []
+        disp1 = self.append_menu_items(
+            (
+                rule.id,
+                MenuItem(
+                    command=action, when=rule.when, group=rule.group, order=rule.order
+                ),
+            )
+            for rule in action.menus or ()
+        )
+        disposers.append(disp1)
+
+        if action.palette:
+            menu_item = MenuItem(command=action, when=action.enablement)
+            disp = self.append_menu_items([(self.COMMAND_PALETTE_ID, menu_item)])
+            disposers.append(disp)
+
+        if not disposers:  # pragma: no cover
+            return None
+
+        def _dispose() -> None:
+            for disposer in disposers:
+                disposer()
+
+        return _dispose
 
     def append_menu_items(
-        self, items: Sequence[Tuple[MenuId, MenuOrSubmenu]]
+        self, items: Iterable[tuple[MenuId, MenuOrSubmenu]]
     ) -> DisposeCallable:
         """Append menu items to the registry.
 
         Parameters
         ----------
-        items : Sequence[Tuple[str, MenuOrSubmenu]]
+        items : Iterable[Tuple[str, MenuOrSubmenu]]
             Items to append.
 
         Returns
@@ -46,10 +76,10 @@ class MenusRegistry:
         DisposeCallable
             A function that can be called to unregister the menu items.
         """
-        changed_ids: Set[str] = set()
-        disposers: List[Callable[[], None]] = []
+        changed_ids: set[str] = set()
+        disposers: list[Callable[[], None]] = []
         for menu_id, item in items:
-            item = MenuItem.validate(item)  # type: ignore
+            item = MenuItem._validate(item)  # type: ignore
             menu_dict = self._menu_items.setdefault(menu_id, {})
             menu_dict[item] = None
             changed_ids.add(menu_id)
@@ -62,9 +92,9 @@ class MenusRegistry:
         def _dispose() -> None:
             for disposer in disposers:
                 disposer()
-            for id in changed_ids:
-                if not self._menu_items.get(id):
-                    del self._menu_items[id]
+            for id_ in changed_ids:
+                if not self._menu_items.get(id_):
+                    del self._menu_items[id_]
             self.menus_changed.emit(changed_ids)
 
         if changed_ids:
@@ -74,13 +104,13 @@ class MenusRegistry:
 
     def __iter__(
         self,
-    ) -> Iterator[Tuple[MenuId, Iterable[MenuOrSubmenu]]]:
+    ) -> Iterator[tuple[MenuId, Iterable[MenuOrSubmenu]]]:
         yield from self._menu_items.items()
 
     def __contains__(self, id: object) -> bool:
         return id in self._menu_items
 
-    def get_menu(self, menu_id: MenuId) -> List[MenuOrSubmenu]:
+    def get_menu(self, menu_id: MenuId) -> list[MenuOrSubmenu]:
         """Return menu items for `menu_id`."""
         # using method rather than __getitem__ so that subclasses can use arguments
         return list(self._menu_items[menu_id])
@@ -92,10 +122,10 @@ class MenusRegistry:
     def __str__(self) -> str:
         return "\n".join(self._render())
 
-    def _render(self) -> List[str]:
+    def _render(self) -> list[str]:
         """Return registered menu items as lines of strings."""
         # this is mostly here as a debugging tool.  Can be removed or improved later.
-        lines: List[str] = []
+        lines: list[str] = []
 
         branch = "  ├──"
         for menu in self._menu_items:
@@ -118,7 +148,7 @@ class MenusRegistry:
             lines.append("")
         return lines
 
-    def iter_menu_groups(self, menu_id: MenuId) -> Iterator[List[MenuOrSubmenu]]:
+    def iter_menu_groups(self, menu_id: MenuId) -> Iterator[list[MenuOrSubmenu]]:
         """Iterate over menu groups for `menu_id`.
 
         Groups are broken into sections (lists of menu or submenu items) based on
@@ -139,12 +169,12 @@ class MenusRegistry:
 
 
 def _sort_groups(
-    items: List[MenuOrSubmenu],
+    items: list[MenuOrSubmenu],
     group_key: Callable = lambda x: "0000" if x == "navigation" else x or "",
     order_key: Callable = lambda x: getattr(x, "order", "") or 0,
-) -> Iterator[List[MenuOrSubmenu]]:
+) -> Iterator[list[MenuOrSubmenu]]:
     """Sort a list of menu items based on their .group and .order attributes."""
-    groups: dict[Optional[str], List[MenuOrSubmenu]] = {}
+    groups: dict[str | None, list[MenuOrSubmenu]] = {}
     for item in items:
         groups.setdefault(item.group, []).append(item)
 
