@@ -1,9 +1,6 @@
 import os
 import sys
 from enum import Enum
-from typing import Callable
-
-DisposeCallable = Callable[[], None]
 
 
 class OperatingSystem(Enum):
@@ -16,7 +13,7 @@ class OperatingSystem(Enum):
 
     @staticmethod
     def current() -> "OperatingSystem":
-        """Returns the current operating system as enum."""
+        """Return the current operating system as enum."""
         return _CURRENT
 
     @property
