@@ -1,10 +1,12 @@
 from enum import IntEnum, IntFlag, auto
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     Dict,
     Generator,
     NamedTuple,
+    Optional,
     Set,
     Tuple,
     Type,
@@ -12,6 +14,12 @@ from typing import (
     overload,
 )
 
+from app_model.types._constants import OperatingSystem
+
+if TYPE_CHECKING:
+    from pydantic.annotated_handlers import GetCoreSchemaHandler
+    from pydantic_core import core_schema
+
 __all__ = ["KeyCode", "KeyMod", "ScanCode", "KeyChord"]
 
 # TODO:
@@ -20,6 +28,7 @@ __all__ = ["KeyCode", "KeyMod", "ScanCode", "KeyChord"]
 # flake8: noqa
 # fmt: off
 
+
 class KeyCode(IntEnum):
     """Virtual Key Codes, the integer value does not hold any inherent meaning.
 
@@ -145,8 +154,35 @@ class KeyCode(IntEnum):
     PauseBreak = auto()
 
     def __str__(self) -> str:
+        """Get a normalized string representation (constant to all OSes) of this `KeyCode`."""
         return keycode_to_string(self)
 
+    def os_symbol(self, os: Optional[OperatingSystem] = None) -> str:
+        """Get a string representation of this `KeyCode` using a symbol/OS specific symbol.
+
+        Some examples:
+            * `KeyCode.Enter` is represented by `↵`
+            * `KeyCode.Meta` is represented by `⊞` on Windows, `Super` on Linux and `⌘` on MacOS
+
+        If no OS is given, the current detected one is used.
+        """
+        os = OperatingSystem.current() if os is None else os
+        return keycode_to_os_symbol(self, os)
+
+    def os_name(self, os: Optional[OperatingSystem] = None) -> str:
+        """Get a string representation of this `KeyCode` using the OS specific naming for the key.
+
+        This differs from `__str__` since with it a normalized representation (constant to all OSes) is given.
+        Sometimes these representations coincide but not always! Some examples:
+            * `KeyCode.Enter` is represented by `Enter` (`__str__` represents it as `Enter`)
+            * `KeyCode.Meta` is represented by `Win` on Windows, `Super` on Linux and `Cmd` on MacOS
+            (`__str__` represents it as `Meta`)
+
+        If no OS is given, the current detected one is used.
+        """
+        os = OperatingSystem.current() if os is None else os
+        return keycode_to_os_name(self, os)
+
     @classmethod
     def from_string(cls, string: str) -> 'KeyCode':
         """Return the `KeyCode` associated with the given string.
@@ -167,6 +203,14 @@ class KeyCode(IntEnum):
     def __get_validators__(cls) -> Generator[Callable[..., 'KeyCode'], None, None]:
         yield cls.validate
 
+    @classmethod
+    def __get_pydantic_core_schema__(
+        cls, source: type, handler: 'GetCoreSchemaHandler'
+    ) -> 'core_schema.CoreSchema':
+        from pydantic_core import core_schema
+
+        return core_schema.no_info_plain_validator_function(cls.validate)
+
     @classmethod
     def validate(cls, value: Any) -> 'KeyCode':
         if isinstance(value, KeyCode):
@@ -183,8 +227,16 @@ class ScanCode(IntEnum):
 
     https://en.wikipedia.org/wiki/Scancode
 
-    These are the scan codea required to conform to the W3C specification for
+    These are the scan codes required to conform to the W3C specification for
     KeyboardEvent.code
+    
+    A scan code is a hardware-specific code that is generated by the keyboard when a key
+    is pressed or released. It represents the physical location of a key on the keyboard
+    and is unique to each key. A key code, on the other hand, is a higher-level
+    representation of a keypress or key release event. They are associated with
+    characters, functions, or actions rather than hardware locations.
+    As an example, the left and right control keys have the same key code (KeyCode.Ctrl)
+    but different scan codes (LeftControl and RightControl).
 
     https://w3c.github.io/uievents-code/
 
@@ -192,6 +244,7 @@ class ScanCode(IntEnum):
     implementations to support special keyboards (such as multimedia or
     legacy keyboards).
     """
+
     UNIDENTIFIED = 0        # This value code should be used when no other value given in this specification is appropriate.
 
     # -----------------------   Writing System Keys   -----------------------
@@ -407,6 +460,8 @@ _NATIVE_WINDOWS_VK_TO_KEYCODE: Dict[str, KeyCode] = {}
 def _build_maps() -> Tuple[
     Callable[[KeyCode], str],
     Callable[[str], KeyCode],
+    Callable[[KeyCode, OperatingSystem], str],
+    Callable[[KeyCode, OperatingSystem], str],
     Callable[[ScanCode], str],
     Callable[[str], ScanCode],
 ]:
@@ -543,7 +598,49 @@ def _build_maps() -> Tuple[
     SCANCODE_FROM_LOWERCASE_STRING: Dict[str, ScanCode] = {}
 
     KEYCODE_TO_STRING: Dict[KeyCode, str] = {}
-    KEYCODE_FROM_LOWERCASE_STRING: Dict[str, KeyCode] = {}
+    KEYCODE_FROM_LOWERCASE_STRING: Dict[str, KeyCode] = {
+        # two special cases for assigning os-specific strings to the meta key
+        'win': KeyCode.Meta,
+        'cmd': KeyCode.Meta,
+    }
+
+    # key symbols on all platforms
+    KEY_SYMBOLS: dict[KeyCode, str] = {
+        KeyCode.Shift: "⇧",
+        KeyCode.LeftArrow: "←",
+        KeyCode.RightArrow: "→",
+        KeyCode.UpArrow: "↑",
+        KeyCode.DownArrow: "↓",
+        KeyCode.Backspace: "⌫",
+        KeyCode.Delete: "⌦",
+        KeyCode.Tab: "⇥",
+        KeyCode.Escape: "⎋",
+        KeyCode.Enter: "↵",
+        KeyCode.Space: "␣",
+        KeyCode.CapsLock: "⇪",
+    }
+    # key symbols mappings per platform
+    OS_KEY_SYMBOLS: dict[OperatingSystem, dict[KeyCode, str]] = {
+        OperatingSystem.WINDOWS: {**KEY_SYMBOLS, KeyCode.Meta: "⊞"},
+        OperatingSystem.LINUX: {**KEY_SYMBOLS, KeyCode.Meta: "Super"},
+        OperatingSystem.MACOS: {
+            **KEY_SYMBOLS,
+            KeyCode.Ctrl: "⌃",
+            KeyCode.Alt: "⌥",
+            KeyCode.Meta: "⌘",
+        },
+    }
+
+    # key names mappings per platform
+    OS_KEY_NAMES: dict[OperatingSystem, dict[KeyCode, str]] = {
+        OperatingSystem.WINDOWS: {KeyCode.Meta: "Win"},
+        OperatingSystem.LINUX: {KeyCode.Meta: "Super"},
+        OperatingSystem.MACOS: {
+            KeyCode.Ctrl: "Control",
+            KeyCode.Alt: "Option",
+            KeyCode.Meta: "Cmd",
+        },
+    }
 
     seen_scancodes: Set[ScanCode] = set()
     seen_keycodes: Set[KeyCode] = set()
@@ -576,6 +673,18 @@ def _build_maps() -> Tuple[
         # sourcery skip
         return KEYCODE_FROM_LOWERCASE_STRING.get(str(keystr).lower(), KeyCode.UNKNOWN)
 
+    def _keycode_to_os_symbol(keycode: KeyCode, os: OperatingSystem) -> str:
+        """Return key symbol for an OS for a given KeyCode."""
+        if keycode in (symbols := OS_KEY_SYMBOLS.get(os, {})):
+            return symbols[keycode]
+        return str(keycode)
+
+    def _keycode_to_os_name(keycode: KeyCode, os: OperatingSystem) -> str:
+        """Return key name for an OS for a given KeyCode."""
+        if keycode in (names := OS_KEY_NAMES.get(os, {})):
+            return names[keycode]
+        return str(keycode)
+
     def _scancode_to_string(scancode: ScanCode) -> str:
         """Return the string representation of a ScanCode."""
         # sourcery skip
@@ -591,6 +700,8 @@ def _build_maps() -> Tuple[
     return (
         _keycode_to_string,
         _keycode_from_string,
+        _keycode_to_os_symbol,
+        _keycode_to_os_name,
         _scancode_to_string,
         _scancode_from_string,
     )
@@ -599,6 +710,8 @@ def _build_maps() -> Tuple[
 (
     keycode_to_string,
     keycode_from_string,
+    keycode_to_os_symbol,
+    keycode_to_os_name,
     scancode_to_string,
     scancode_from_string,
 ) = _build_maps()
@@ -629,16 +742,13 @@ class KeyMod(IntFlag):
     WinCtrl = 1 << 8  # meta key on windows, ctrl key on mac
 
     @overload  # type: ignore
-    def __or__(self, other: "KeyMod") -> "KeyMod":
-        ...
+    def __or__(self, other: "KeyMod") -> "KeyMod": ...
 
     @overload
-    def __or__(self, other: KeyCode) -> "KeyCombo":
-        ...
+    def __or__(self, other: KeyCode) -> "KeyCombo": ...
 
     @overload
-    def __or__(self, other: int) -> int:
-        ...
+    def __or__(self, other: int) -> int: ...
 
     def __or__(
         self, other: Union["KeyMod", KeyCode, int]
@@ -651,8 +761,10 @@ class KeyMod(IntFlag):
 
 
 class KeyCombo(int):
-    """KeyCombo is an integer combination of one or more
-    [`KeyMod`][app_model.types.KeyMod] and [`KeyCode`][app_model.types.KeyCode]."""
+    """KeyCombo is an integer combination of one or more.
+
+    [`KeyMod`][app_model.types.KeyMod] and [`KeyCode`][app_model.types.KeyCode].
+    """
 
     def __new__(
         cls: Type["KeyCombo"], modifiers: KeyMod, key: KeyCode = KeyCode.UNKNOWN
@@ -670,8 +782,18 @@ class KeyCombo(int):
 
 
 class KeyChord(int):
-    """KeyChord is an integer combination of two [`KeyCombo`][app_model.types.KeyCombo],
-    [`KeyCode`][app_model.types.KeyCode], or [int][]."""
+    """KeyChord is an integer combination of two key combos.
+
+    It could be two [`KeyCombo`][app_model.types.KeyCombo]
+    [`KeyCode`][app_model.types.KeyCode], or [int][].
+
+    Parameters
+    ----------
+    first_part : KeyCombo | int
+        The first part of the chord.
+    second_part : KeyCombo | int
+        The second part of the chord.
+    """
 
     def __new__(cls: Type["KeyChord"], first_part: int, second_part: int) -> "KeyChord":
         # shift the second part 16 bits to the left
